==> cbmc_5.9.orig.tar.gz: cbmc-cbmc-5.9/src/goto-instrument/cover_instrument_other.cpp <==
/*******************************************************************\

Module: Coverage Instrumentation

Author: Daniel Kroening

\*******************************************************************/

/// \file
/// Further coverage instrumentations

#include "cover_instrument.h"

#include <langapi/language_util.h>

#include "cover_util.h"

void cover_path_instrumentert::instrument(
  goto_programt &goto_program,
  goto_programt::targett &i_it,
  const cover_blocks_baset &basic_blocks) const
{
  if(is_non_cover_assertion(i_it))
    i_it->make_skip();

  // TODO: implement
}

void cover_assertion_instrumentert::instrument(
  goto_programt &goto_program,
  goto_programt::targett &i_it,
  const cover_blocks_baset &basic_blocks) const
{
  // turn into 'assert(false)' to avoid simplification
  if(is_non_cover_assertion(i_it))
  {
    i_it->guard = false_exprt();
    initialize_source_location(
      i_it, id2string(i_it->source_location.get_comment()), i_it->function);
  }
}

void cover_cover_instrumentert::instrument(
  goto_programt &goto_program,
  goto_programt::targett &i_it,
  const cover_blocks_baset &basic_blocks) const
{
  // turn __CPROVER_cover(x) into 'assert(!x)'
  if(i_it->is_function_call())
  {
    const code_function_callt &code_function_call =
      to_code_function_call(i_it->code);
    if(
      code_function_call.function().id() == ID_symbol &&
      to_symbol_expr(code_function_call.function()).get_identifier() ==
        "__CPROVER_cover" &&
      code_function_call.arguments().size() == 1)
    {
      const exprt c = code_function_call.arguments()[0];
      std::string comment =
        "condition `" + from_expr(ns, i_it->function, c) + "'";
      i_it->guard = not_exprt(c);
      i_it->type = ASSERT;
      i_it->code.clear();
      initialize_source_location(i_it, comment, i_it->function);
    }
  }
  else if(is_non_cover_assertion(i_it))
    i_it->make_skip();
}

void cover_instrument_end_of_function(
  const irep_idt &function,
  goto_programt &goto_program)
{
  auto if_it = goto_program.instructions.end();
  while(!if_it->is_function_call())
    if_it--;
  if_it++;
  const std::string &comment =
    "additional goal to ensure reachability of end of function";
  goto_program.insert_before_swap(if_it);
  if_it->make_assertion(false_exprt());
  if_it->source_location.set_comment(comment);
  if_it->source_location.set_property_class("reachability_constraint");
  if_it->source_location.set_function(function);
  if_it->function = function;
}

==> cbmc_5.10.orig.tar.gz: cbmc-cbmc-5.10/src/goto-instrument/cover_instrument_other.cpp <==
/*******************************************************************\

Module: Coverage Instrumentation

Author: Daniel Kroening

\*******************************************************************/

/// \file
/// Further coverage instrumentations

#include "cover_instrument.h"

#include <langapi/language_util.h>

#include "cover_util.h"

void cover_path_instrumentert::instrument(
  goto_programt &,
  goto_programt::targett &i_it,
  const cover_blocks_baset &) const
{
  if(is_non_cover_assertion(i_it))
    i_it->make_skip();

  // TODO: implement
}

void cover_assertion_instrumentert::instrument(
  goto_programt &,
  goto_programt::targett &i_it,
  const cover_blocks_baset &) const
{
  // turn into 'assert(false)' to avoid simplification
  if(is_non_cover_assertion(i_it))
  {
    i_it->guard = false_exprt();
    initialize_source_location(
      i_it, id2string(i_it->source_location.get_comment()), i_it->function);
  }
}

void cover_cover_instrumentert::instrument(
  goto_programt &,
  goto_programt::targett &i_it,
  const cover_blocks_baset &) const
{
  // turn __CPROVER_cover(x) into 'assert(!x)'
  if(i_it->is_function_call())
  {
    const code_function_callt &code_function_call =
      to_code_function_call(i_it->code);
    if(
      code_function_call.function().id() == ID_symbol &&
      to_symbol_expr(code_function_call.function()).get_identifier() ==
        "__CPROVER_cover" &&
      code_function_call.arguments().size() == 1)
    {
      const exprt c = code_function_call.arguments()[0];
      std::string comment =
        "condition `" + from_expr(ns, i_it->function, c) + "'";
      i_it->guard = not_exprt(c);
      i_it->type = ASSERT;
      i_it->code.clear();
      initialize_source_location(i_it, comment, i_it->function);
    }
  }
  else if(is_non_cover_assertion(i_it))
    i_it->make_skip();
}

void cover_instrument_end_of_function(
  const irep_idt &function,
  goto_programt &goto_program)
{
  auto if_it = goto_program.instructions.end();
  while(!if_it->is_function_call())
    if_it--;
  if_it++;
  const std::string &comment =
    "additional goal to ensure reachability of end of function";
  goto_program.insert_before_swap(if_it);
  if_it->make_assertion(false_exprt());
  if_it->source_location.set_comment(comment);
  if_it->source_location.set_property_class("reachability_constraint");
  if_it->source_location.set_function(function);
  if_it->function = function;
}
